@@ -47,27 +47,23 @@ class Plottable(object):
     _bolt_driver : Any
     _tigergraph : Any
 
-    _node_embedding : Optional[Any]
+    _node_embedding : Optional[pd.DataFrame]
     _node_encoder : Optional[Any]
     _node_features : Optional[pd.DataFrame]
-    _node_ordinal_pipeline: Optional[Pipeline]
     _node_target : Optional[pd.DataFrame]
-    _node_target_encoder : Optional[Any]
 
-    _edge_embedding : Optional[Any]
-    _edge_encoders : Optional[Any]
+    _edge_embedding : Optional[pd.DataFrame]
+    _edge_encoder : Optional[Any]
     _edge_features : Optional[pd.DataFrame]
-    _edge_ordinal_pipeline : Optional[Pipeline]
     _edge_target : Optional[pd.DataFrame]
-    _edge_target_encoder : Optional[Any]
 
     _weighted_adjacency: Optional[Any]
     _weighted_adjacency_nodes : Optional[Any]
     _weighted_adjacency_edges : Optional[Any]
-    _weighted_edges_df : Optional[Any]
-    _weighted_edges_df_from_nodes : Optional[Any]
-    _weighted_edges_df_from_edges : Optional[Any]
-    _xy: Optional[Any]
+    _weighted_edges_df : Optional[pd.DataFrame]
+    _weighted_edges_df_from_nodes : Optional[pd.DataFrame]
+    _weighted_edges_df_from_edges : Optional[pd.DataFrame]
+    _xy: Optional[pd.DataFrame]
 
     _umap : Optional[UMAP]
 
@@ -75,6 +71,7 @@ class Plottable(object):
     _entity_to_index : dict
     _index_to_entity : dict
 
+    DGL_graph: Optional[Any]
 
     def __init__(self, *args, **kwargs):
         #raise RuntimeError('should not happen')
@@ -147,7 +144,8 @@ class Plottable(object):
         unwrap: bool = False,
         verbose: bool = False
     ) -> 'Plottable':
-        raise RuntimeError('should not happen')
+        if 1 + 1:
+            raise RuntimeError('should not happen')
         return self
 
     def hop(self,
@@ -160,15 +158,18 @@ class Plottable(object):
         destination_node_match: Optional[dict] = None,
         return_as_wave_front: bool = False
     ) -> 'Plottable':
-        raise RuntimeError('should not happen')
+        if 1 + 1:
+            raise RuntimeError('should not happen')
         return self
 
     def filter_nodes_by_dict(self, filter_dict: Optional[dict] = None) -> 'Plottable':
-        raise RuntimeError('should not happen')
+        if 1 + 1:
+            raise RuntimeError('should not happen')
         return self
 
     def filter_edges_by_dict(self, filter_dict: Optional[dict] = None) -> 'Plottable':
-        raise RuntimeError('should not happen')
+        if 1 + 1:
+            raise RuntimeError('should not happen')
         return self
 
     # FIXME python recursive typing issues
@@ -176,5 +177,27 @@ class Plottable(object):
         """
         ops is List[ASTObject]
         """
-        raise RuntimeError('should not happen')
+        if 1 + 1:
+            raise RuntimeError('should not happen')
+        return self
+
+    def to_igraph(self, 
+        directed: bool = True,
+        include_nodes: bool = True,
+        node_attributes: Optional[List[str]] = None,
+        edge_attributes: Optional[List[str]] = None
+    ) -> Any:
+        if 1 + 1:
+            raise RecursionError('should not happen')
+        return self
+
+    def from_igraph(self,
+        ig,
+        node_attributes: Optional[List[str]] = None,
+        edge_attributes: Optional[List[str]] = None,
+        load_nodes: bool = True, load_edges: bool = True,
+        merge_if_existing: bool = True
+    ):
+        if 1 + 1:
+            raise RecursionError('should not happen')
         return self
