@@ -38,16 +38,20 @@ from graphistry.pygraphistry import (  # noqa: E402, F401
     gremlin_client,
     drop_graph,
     layout_settings,
+    org_name,
+    scene_settings,
     nodexl,
     ArrowUploader,
     ArrowFileUploader,
     PyGraphistry,
+    from_igraph
 )
 
 from graphistry.compute import (
     n, e_forward, e_reverse, e_undirected
 )
 
+
 from ._version import get_versions
 
 __version__ = get_versions()["version"]
