@@ -1,9 +1,11 @@
 # ###############################################################
-VERBOSE = False  # set to true for debug
+VERBOSE = None  # set to true for info, false for debug, None for none
+TRACE = False # set to true for full trace of functions
 # ###############################################################
 # source and destination labels for consistent pipeline-ing across files
 SRC = "_src_implicit"
 DST = "_dst_implicit"
+NODE = '_n_implicit'
 WEIGHT = "_weight"
 # for UMAP reserved namespace
 X = "x"
@@ -30,6 +32,7 @@ TEST_MASK = "test_mask"
 #   for dirty_cat params
 DIRTY_CAT = "dirty_cat"
 N_TOPICS_DEFAULT = 42
+N_TOPICS_TARGET_DEFAULT = 7
 N_HASHERS_DEFAULT = 100
 
 # scikit-learn params
